import grok.tests.grokker.priority

class AlphaSub(grok.tests.grokker.priority.Alpha):
    pass

class BetaSub(grok.tests.grokker.priority.Beta):
    pass

class GammaSub(grok.tests.grokker.priority.Gamma):
    pass
